==> shelf_route-0.1.1%2B1.tar.gz: test/shelf_route_test.dart <==
// Copyright (c) 2014, The Shelf Route project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_route.test;

import 'package:shelf/shelf.dart';
import 'package:shelf_route/shelf_route.dart' as r;

import 'package:unittest/unittest.dart';
import 'package:mock/mock.dart';

Request _request([String pathInfo = "/", String scriptName = ""]) {
  return new Request(pathInfo, "", "GET", scriptName, "1.1",
      Uri.parse('http://localhost/'), {});
}

//typedef Handler(Request request);
abstract class HandlerClass {
  dynamic call(Request request) {}
}

class MockHandler extends Mock implements HandlerClass {
  noSuchMethod(Invocation invocation) => super.noSuchMethod(invocation);
}

void main() {
  group('when no routes are defined', () {
    test('an immediate response is returned', () {
      expect(r.router().handler(_request()), new isInstanceOf<Response>());
    });

    test('status code is 404', () {
      expect((r.router().handler(_request()) as Response).statusCode, equals(404));
    });
  });

  group('when one route is defined', () {
    MockHandler handler;
    r.Router router;

    setUp(() {
      handler = new MockHandler();
      router = r.router();
    });

    Handler _routeHandler(String path) {
      router.addRoute(handler, path: path);
      return router.handler;
    }

    Response _routeRequest(String routePath, String requestPath) =>
        _routeHandler(routePath)(_request(requestPath));


    test('then a null path and method matches all', () {
      _routeRequest(null, '/');
      handler.calls('call').verify(happenedOnce);
    });

    test('then a / path and null method matches all', () {
      _routeRequest('/', '/');
      handler.calls('call').verify(happenedOnce);
    });

    test('then an empty path and null method matches all', () {
      _routeRequest('', '/');
      handler.calls('call').verify(happenedOnce);
    });

    group('then when path same as pathInfo', () {
      final String path = '/foo/bar';

      test('route is matched', () {
        _routeRequest(path, path);
        handler.calls('call').verify(happenedOnce);
      });

      test('new pathInfo is empty', () {
        _routeRequest(path, path);
        handler.calls('call', new HasPathInfo(isEmpty)).verify(happenedOnce);
      });

      test('new scriptName is old pathInfo', () {
        _routeRequest(path, path);
        handler.calls('call', new HasScriptName(equals(path)))
          .verify(happenedOnce);
      });
    });


    group('then when route path is parent of request pathInfo', () {
      final String routePath = '/foo/bar';
      final String subPath = '/fum/blah';
      final String requestPath = '$routePath$subPath';

      test('route is matched', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call').verify(happenedOnce);
      });

      test('new pathInfo is the sub path of the route', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasPathInfo(equals(subPath))).verify(happenedOnce);
      });

      test('new scriptName is old route path', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasScriptName(equals(routePath)))
          .verify(happenedOnce);
      });
    });

    group('then when route path is not parent of request pathInfo', () {
      final String routePath = '/foo/bar';
      final String requestPath = '/fool/bar/blah';

      test('route is matched', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call').verify(neverHappened);
      });

      test('status code is 404', () {
        expect(_routeRequest(routePath, requestPath).statusCode, equals(404));
      });
    });

  });

  group('when routes are nested', () {
    MockHandler handler;
    r.Router parentRouter;
    r.Router router;
    const String parentPath = '/firstseg';

    setUp(() {
      handler = new MockHandler();
      parentRouter = r.router();
      router = parentRouter.addChildRouter(parentPath);
    });

    Handler _routeHandler(String path) {
      router.addRoute(handler, path: path);
      return parentRouter.handler;
    }

    Response _routeRequest(String routePath, String requestPath) =>
        _routeHandler(routePath)(_request(requestPath));

    group('then when combined paths are same as pathInfo', () {
      final String routePath = '/bar';
      final String requestPath = '$parentPath$routePath';

      test('route is matched', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call').verify(happenedOnce);
      });

      test('new pathInfo is empty', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasPathInfo(isEmpty)).verify(happenedOnce);
      });

      test('new scriptName is old pathInfo', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasScriptName(equals(requestPath)))
          .verify(happenedOnce);
      });
    });

    group('then when final route is /', () {
      final String routePath = '/';
      final String requestPath = '$parentPath$routePath';

      test('route is matched', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call').verify(happenedOnce);
      });

      test('new pathInfo is empty', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasPathInfo(isEmpty)).verify(happenedOnce);
      });

      test('new scriptName is old pathInfo excluding trailing /', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasScriptName(equals(parentPath)))
          .verify(happenedOnce);
      });
    });
  });

  group('when path contains a pathVariable', () {
    MockHandler handler;
    r.Router router;

    setUp(() {
      handler = new MockHandler();
      router = r.router();
    });

    Handler _routeHandler(String path) {
      router.addRoute(handler, path: path);
      return router.handler;
    }

    Response _routeRequest(String routePath, String requestPath) =>
        _routeHandler(routePath)(_request(requestPath));


    group('of type String', () {
      final String routePath = '/person/:name';
      final String requestPath = '/person/fred';

      test('the pathVariable acts like a wildcard for a path segment', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasPathInfo(isEmpty))
          .verify(happenedOnce);
        handler.calls('call', new HasScriptName(equals(requestPath)))
          .verify(happenedOnce);
      });

      test('the value of the pathVariable is included in the request passed to handler', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasHeaderValue('shelf.route.name',
            equals('fred'))).verify(happenedOnce);
      });
    });

  });
}

class HasPathInfo extends CustomMatcher {
  HasPathInfo(Matcher matcher)
    : super("Request with pathInfo that is", "pathInfo", matcher);

  String featureValueOf(Request actual) => actual.pathInfo;
}

class HasScriptName extends CustomMatcher {
  HasScriptName(Matcher matcher)
    : super("Request with scriptName that is", "scriptName", matcher);

  String featureValueOf(Request actual) => actual.scriptName;
}

class HasHeaderValue extends CustomMatcher {
  final String headerName;
  HasHeaderValue(String headerName, Matcher matcher)
    : this.headerName = headerName,
      super("Request with header whose key is '$headerName' and value that is",
          "headers", matcher);

  String featureValueOf(Request actual) => actual.headers[headerName];
}

==> shelf_route-0.2.0%2B1.tar.gz: test/shelf_route_test.dart <==
// Copyright (c) 2014, The Shelf Route project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_route.test;

import 'package:shelf/shelf.dart';
import 'package:shelf_route/shelf_route.dart' as r;

import 'package:unittest/unittest.dart';
import 'package:mock/mock.dart';

Request _request([String pathInfo = "/", String scriptName = ""]) {
  return new Request("GET", Uri.parse('http://localhost/'),
      protocolVersion: "1.1",
      headers: {}, url: Uri.parse(pathInfo), scriptName: scriptName);
}

//typedef Handler(Request request);
abstract class HandlerClass {
  dynamic call(Request request) { return null; }
}

class MockHandler extends Mock implements HandlerClass {
  noSuchMethod(Invocation invocation) => super.noSuchMethod(invocation);
}

void main() {
  group('when no routes are defined', () {
    test('an immediate response is returned', () {
      expect(r.router().handler(_request()), new isInstanceOf<Response>());
    });

    test('status code is 404', () {
      expect((r.router().handler(_request()) as Response).statusCode, equals(404));
    });
  });

  group('when one route is defined', () {
    MockHandler handler;
    r.Router router;

    setUp(() {
      handler = new MockHandler();
      router = r.router();
    });

    Handler _routeHandler(String path) {
      router.addRoute(handler, path: path);
      return router.handler;
    }

    Response _routeRequest(String routePath, String requestPath) =>
        _routeHandler(routePath)(_request(requestPath));


    test('then a null path and method matches all', () {
      _routeRequest(null, '/');
      handler.calls('call').verify(happenedOnce);
    });

    test('then a / path and null method matches all', () {
      _routeRequest('/', '/');
      handler.calls('call').verify(happenedOnce);
    });

    test('then an empty path and null method matches all', () {
      _routeRequest('', '/');
      handler.calls('call').verify(happenedOnce);
    });

    group('then when path same as pathInfo', () {
      final String path = '/foo/bar';

      test('route is matched', () {
        _routeRequest(path, path);
        handler.calls('call').verify(happenedOnce);
      });

      test('new pathInfo is empty', () {
        _routeRequest(path, path);
        handler.calls('call', new HasPathInfo(isEmpty)).verify(happenedOnce);
      });

      test('new scriptName is old pathInfo', () {
        _routeRequest(path, path);
        handler.calls('call', new HasScriptName(equals(path)))
          .verify(happenedOnce);
      });
    });


    group('then when route path is parent of request pathInfo', () {
      final String routePath = '/foo/bar';
      final String subPath = '/fum/blah';
      final String requestPath = '$routePath$subPath';

      test('route is matched', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call').verify(happenedOnce);
      });

      test('new pathInfo is the sub path of the route', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasPathInfo(equals(subPath))).verify(happenedOnce);
      });

      test('new scriptName is old route path', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasScriptName(equals(routePath)))
          .verify(happenedOnce);
      });
    });

    group('then when route path is not parent of request pathInfo', () {
      final String routePath = '/foo/bar';
      final String requestPath = '/fool/bar/blah';

      test('route is matched', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call').verify(neverHappened);
      });

      test('status code is 404', () {
        expect(_routeRequest(routePath, requestPath).statusCode, equals(404));
      });
    });

  });

  group('when routes are nested', () {
    MockHandler handler;
    r.Router parentRouter;
    r.Router router;
    const String parentPath = '/firstseg';

    setUp(() {
      handler = new MockHandler();
      parentRouter = r.router();
      router = parentRouter.addChildRouter(parentPath);
    });

    Handler _routeHandler(String path) {
      router.addRoute(handler, path: path);
      return parentRouter.handler;
    }

    Response _routeRequest(String routePath, String requestPath) =>
        _routeHandler(routePath)(_request(requestPath));

    group('then when combined paths are same as pathInfo', () {
      final String routePath = '/bar';
      final String requestPath = '$parentPath$routePath';

      test('route is matched', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call').verify(happenedOnce);
      });

      test('new pathInfo is empty', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasPathInfo(isEmpty)).verify(happenedOnce);
      });

      test('new scriptName is old pathInfo', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasScriptName(equals(requestPath)))
          .verify(happenedOnce);
      });
    });

    group('then when final route is /', () {
      final String routePath = '/';
      final String requestPath = '$parentPath$routePath';

      test('route is matched', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call').verify(happenedOnce);
      });

      test('new pathInfo is empty', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasPathInfo(isEmpty)).verify(happenedOnce);
      });

      test('new scriptName is old pathInfo excluding trailing /', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasScriptName(equals(parentPath)))
          .verify(happenedOnce);
      });
    });
  });

  group('when path contains a pathVariable', () {
    MockHandler handler;
    r.Router router;

    setUp(() {
      handler = new MockHandler();
      router = r.router();
    });

    Handler _routeHandler(String path) {
      router.addRoute(handler, path: path);
      return router.handler;
    }

    Response _routeRequest(String routePath, String requestPath) =>
        _routeHandler(routePath)(_request(requestPath));


    group('of type String', () {
      final String routePath = '/person/:name';
      final String requestPath = '/person/fred';

      test('the pathVariable acts like a wildcard for a path segment', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasPathInfo(isEmpty))
          .verify(happenedOnce);
        handler.calls('call', new HasScriptName(equals(requestPath)))
          .verify(happenedOnce);
      });

      test('the value of the pathVariable is available via getPathVariables', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call',
            new HasPathVariables(containsPair('name', 'fred')))
              .verify(happenedOnce);
      });

    });

    group('as the only segment', () {
      final String routePath = '/:name';
      final String requestPath = '/fred';

      test('the pathVariable acts like a wildcard for a path segment', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call', new HasPathInfo(isEmpty))
          .verify(happenedOnce);
        handler.calls('call', new HasScriptName(equals(requestPath)))
          .verify(happenedOnce);
      });

      test('the value of the pathVariable is available via getPathVariables', () {
        _routeRequest(routePath, requestPath);
        handler.calls('call',
            new HasPathVariables(containsPair('name', 'fred')))
              .verify(happenedOnce);
      });

    });
  });
}

class HasPathInfo extends CustomMatcher {
  HasPathInfo(Matcher matcher)
    : super("Request with pathInfo that is", "pathInfo", matcher);

  String featureValueOf(Request actual) => actual.url.path;
}

class HasScriptName extends CustomMatcher {
  HasScriptName(Matcher matcher)
    : super("Request with scriptName that is", "scriptName", matcher);

  String featureValueOf(Request actual) => actual.scriptName;
}

class HasHeaderValue extends CustomMatcher {
  final String headerName;
  HasHeaderValue(String headerName, Matcher matcher)
    : this.headerName = headerName,
      super("Request with header whose key is '$headerName' and value that is",
          "headers", matcher);

  String featureValueOf(Request actual) => actual.headers[headerName];
}

class HasPathVariables extends CustomMatcher {
  HasPathVariables(Matcher matcher)
    : super("Request with path variables that contain",
          "headers", matcher);

  Map<String, Object> featureValueOf(Request actual) =>
      r.getPathVariables(actual);
}
